{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE TypeOperators #-}
{-# LANGUAGE DataKinds #-}
{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE TemplateHaskell #-}

module ISTEX.Client where

import GHC.Generics
import Data.Aeson
import Control.Applicative ((<|>))

import Servant.API
import Servant.Client
import Data.Proxy (Proxy(..))

import qualified Data.Text as T
import qualified Control.Lens as L

data Author = Author
  {
    _author_name :: T.Text,
    _author_affiliations :: [T.Text]
  } deriving (Show, Generic)
L.makeLenses ''Author

instance FromJSON Author where
  parseJSON (Object o) =
    Author <$> (o .: "name") <*> (o .: "affiliations" <|> pure [])

data Source = Source
  {
    _source_title :: Maybe T.Text,
    _source_authors :: [Author],
    _source_publicationDate :: Maybe Int
  } deriving (Show, Generic)
L.makeLenses ''Source

instance FromJSON Source where
  parseJSON (Object o) =
    Source <$>
    (o .:? "title")
    <*> (o .: "author")
    <*> do pPubDate <- (o .:? "publicationDate")
           return $ (read . T.unpack) <$> pPubDate

data Document = Document
  {
    _document_id :: T.Text,
    _document_title :: Maybe T.Text,
    _document_authors :: [Author],
    _document_abstract :: Maybe T.Text,
    _document_publicationDate :: Maybe Int,
    _document_sources :: [Source]
  } deriving (Show, Generic)
L.makeLenses ''Document

instance FromJSON Document where
  parseJSON (Object o) =
    Document <$>
    (o .: "id")
    <*> (o .:? "title")
    <*> (o .: "author" <|> pure [])
    <*> (o .:? "abstract")
    <*> do pPubDate <- (o .:? "publicationDate")
           return $ (read . T.unpack) <$> pPubDate
    <*> (o .: "refBibs" <|> pure [])

data Documents = Documents
  {
    _documents_total :: Int,
    _documents_hits :: [Document]
  } deriving (Show, Generic)
L.makeLenses ''Documents
instance FromJSON Documents where
  parseJSON (Object o) =
    Documents <$> (o .: "total") <*> (o .: "hits")

type ISTEXAPI = Search

type Search =
  QueryParam "output" T.Text
  :> QueryParam "size" Int
  :> QueryParam "q" T.Text
  :> Get '[JSON] Documents

istexProxy :: Proxy (ISTEXAPI)
istexProxy = Proxy

search :: Maybe T.Text -> Maybe Int -> Maybe T.Text -> ClientM Documents
search = client istexProxy
